/* Copyright 2005-2006 Michelle Osmond
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

package message;

import java.util.*;

import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.PortletSessionUtil;
import javax.portlet.PortletPreferences;


/**
 *  This is intended as the main interface between portlet code
 *  and the messaging system. It can be used to send and receive
 *  messages, and to load/save message mappings between the portlet 
 *  preferences and the session. 
 *  Each portlet should ensure that loadPrefs is called at least once
 *  per session (for example at the beginning of doView).
 *  
 * @author Michelle Osmond
 * @version 1.2
 */
public class MessageHelper implements MessageConstants {

	protected MessageCentre msgCentre = null;
	protected String portletID = "";
    protected String sessionID = "";

    /** Retrieve MessageHelper for messaging between portlets in one or more webapps.
     *  The portletID should be an identifier for the portlet instance that is unique
     *  across all portlet apps, if you want to communicate between webapps.
     *  The sessionID should be an identifier to represent a particular user's browser 
     *  session, that is shared across all portlet apps.
     */
	public MessageHelper(PortletSession session, String portletID, String sessionID){
		this.portletID = portletID;
		this.msgCentre = MessageHelper.getMessageCentre(session, sessionID);
        this.sessionID = sessionID;
	}

    /** Retrieve MessageHelper. 
     *  This will automatically use MessageHelper.getPortletID and MessageHelper.getSessionID.
     */
    public MessageHelper(PortletRequest request){
        this(request.getPortletSession(), MessageHelper.getPortletID(request), MessageHelper.getSessionID(request));
    }

    /** Retrieve MessageCentre for messaging between portlets in one or more webapps. 
     *  The MessageStore implementation used can be configured.
     *  This checks MessageCentre.isValid() before returning it; if it is not valid,
     *  it creates a new one.
     */
	public static MessageCentre getMessageCentre(PortletSession session, String sessionID){
		Object obj = session.getAttribute(MSG_CENTRE_KEY, PortletSession.APPLICATION_SCOPE);
		if (obj!=null && obj instanceof MessageCentre){
            if ( ((MessageCentre)obj).isValid() ){
    			return (MessageCentre) obj;
            }
		}
        try {
            // Create a new message centre
    		MessageCentre newCentre = MessagingFactory.getMessageCentre(sessionID);
    		session.setAttribute(MSG_CENTRE_KEY, newCentre, PortletSession.APPLICATION_SCOPE);
            // Also create a new loaded-prefs tracker
			session.setAttribute(MSG_LOADED, new TreeMap(), PortletSession.APPLICATION_SCOPE);
    		return newCentre;
        } catch (Exception e){
            System.out.println("ERROR - there was a problem finding or instantiating the class for MessageCentre!");
            e.printStackTrace();
            return null;
        }
	}

	/** Utility: get (and set, if not yet present) a unique ID for this 
	 *  portlet instance, cached in the session.
     *  It will try to retrieve the true window ID assigned by the portal, 
     *  and then prefix it with the portlet context name. 
     *  So IDs should be unique across portlet applications
     *  and probably persistent across sessions.
     */
    public static String getPortletID(PortletRequest request){
        return MessageHelper.getPortletID(request, true);
    }

	/** Utility: get (and set, if not yet present) a unique ID for this 
	 *  portlet instance, cached in the session.
     *  This uses the portlet context name, so IDs should be unique
     *  across portlet applications.
     *  If useWindowID is set, it tries to retrieve the actual portlet window ID
     *  assigned by the portal. If not set, it generates a random number for the ID.
     */
    public static String getPortletID(PortletRequest request, boolean useWindowID){
        // see if portlet id has been set in session.
        PortletSession session = request.getPortletSession(true);
        if (session==null || session.getAttribute(MSG_PORTLET_ID)==null || session.getAttribute(MSG_PORTLET_ID).toString().equals("")){
            // no id set; generate a new one
            String id = "";
            if (useWindowID){   
                id = MessageHelper.getPortletWindowID(request);
            }
            if (id==null || id.length()==0){
                // make a random ID
                double random = MessageHelper.getRandomNumber();
                id = ""+random;
            }
            String context_name = request.getContextPath();
            session.setAttribute(MSG_PORTLET_ID, context_name+"_"+id);            
        }
        return session.getAttribute(MSG_PORTLET_ID).toString();
    }

	/** Utility: Try to find the real portlet window ID, by setting a temporary
     *  attribute in the local portlet session.
     *  Returns null if unable to do so.
     */
    public static String getPortletWindowID(PortletRequest request){
        String windowID = null;

        // 1. set a PORTLET_SCOPE attribute with a known name, e.g. "retrieveID". 
        // To ensure that there will be no interference from other portlets doing the same, 
        // randomly generate part of this name.
        double random = MessageHelper.getRandomNumber();
        String att_name = "retrieveID"+random;

        PortletSession session = request.getPortletSession(true);
        if (session==null){ return null; }
        session.setAttribute(att_name, "test", PortletSession.PORTLET_SCOPE);

        // 2. get a list of the attributes in the session, in the APPLICATION_SCOPE
        Enumeration names_enum = session.getAttributeNames(PortletSession.APPLICATION_SCOPE);
        
        // 3. find the full (namespaced) name of the attribute that was just set, 
        // using PortletSessionUtil to find the one containing the PORTLET_SCOPE name
        while ( names_enum.hasMoreElements() ) {
            String name = (String) names_enum.nextElement();
            if (PortletSessionUtil.decodeScope(name)==PortletSession.PORTLET_SCOPE){
                String local_name = PortletSessionUtil.decodeAttributeName(name);
                if (att_name.equals(local_name)){
                    // Found the attribute we set.
                    // break down this full name into parts as described in PLT.15.3 
                    // to extract the Window ID: the name should be of the form
                    // "javax.portlet.p.<ID>?<ATTRIBUTE_NAME>"
                    String prefix = "javax.portlet.p.";
                    String suffix = "?"+att_name;
                    if (name.startsWith(prefix) && name.endsWith(suffix)){
                        windowID = name.substring(prefix.length(), name.length() - suffix.length());
                    }
                    if (MessageConfig.debug()){
                        System.out.println("Name of PORTLET_SCOPE attribute: ["+att_name+"]");
                        System.out.println("Name of APPLICATION_SCOPE attribute: ["+name+"]");
                        System.out.println(" - Window ID: ["+windowID+"]");
                    }
                }
            }            
        }
        
        // 4. clean up: remove the PORTLET_SCOPE attribute that was set earlier
        session.removeAttribute(att_name, PortletSession.PORTLET_SCOPE);

        return windowID;
    }

    private static double getRandomNumber(){
        double n1 = Math.random();
        double n2 = Math.random();
        double random = n1 / n2;
        return random;
    }

	/** Utility: get a browser session ID. This should be available across portlet
     *  applications if you want cross context communication. 
     *  The SessionIDRetriever used can be configured.
     */
    public static String getSessionID(PortletRequest request){
		Object obj = request.getPortletSession(true).getAttribute(MSG_SESSION_ID_RETRIEVER, PortletSession.APPLICATION_SCOPE);
		if (obj!=null && obj instanceof SessionIDRetriever){
			return ((SessionIDRetriever)obj).getSessionID(request);
		}
        try {
            SessionIDRetriever retriever = MessagingFactory.getSessionIDRetriever();
            request.getPortletSession(true).setAttribute(MSG_SESSION_ID_RETRIEVER, retriever, PortletSession.APPLICATION_SCOPE);
            return retriever.getSessionID(request);
        } catch (Exception e){
            System.out.println("ERROR - there was a problem finding or instantiating the class for SessionIDRetriever!");
            e.printStackTrace();
            return null;
        }
    }

	protected static boolean loadedPrefs(PortletSession session, String portletID){
		Set loaded = getLoadedPrefs(session);
		return loaded.contains(portletID);
	}

	protected static Set getLoadedPrefs(PortletSession session){
		Object obj = session.getAttribute(MSG_LOADED, PortletSession.APPLICATION_SCOPE);
		if (obj==null || !(obj instanceof Set)){
			obj = new TreeSet();
			session.setAttribute(MSG_LOADED, obj, PortletSession.APPLICATION_SCOPE);
		}
		return (Set) obj;
	}

	/**
	 *  Sets a marker for whether or not the message mappings in the 
     *  specified portlet's preferences have been loaded into the session.
     *  You can force <a href="#loadPrefs">loadPrefs</a> to reload the
     *  preferences by first calling this with isLoaded=false.
	 */
	public static void setLoadedPrefs(PortletSession session, String portletID, boolean isLoaded){
		Set loaded = getLoadedPrefs(session);
		if (isLoaded){
			loaded.add(portletID);
		} else {
			loaded.remove(portletID);
		}
	}

	/**
	 *  Loads portlet preferences for message mappings into the session,
     *  if this has not already been done (checked using <a href="#loadedPrefs">loadedPrefs</a>).
     *  This automatically calls getPortletID and getSessionID on the provided request.
	 */
	public static void loadPrefs(PortletRequest request){
        MessageHelper.loadPrefs(request, MessageHelper.getPortletID(request), MessageHelper.getSessionID(request));
    }

	/**
	 *  Loads portlet preferences for message mappings into the session,
     *  if this has not already been done (checked using <a href="#loadedPrefs">loadedPrefs</a>).
	 */
	public static void loadPrefs(PortletRequest request, String portletID, String sessionID){
        boolean debug = MessageConfig.debug();
        if (debug){
    		System.out.println("MessageHelper: Seeing if we need to load prefs for portlet "+portletID);
        }
		PortletSession portletSession = request.getPortletSession(true);
		MessageCentre msgCentre = MessageHelper.getMessageCentre(portletSession, sessionID);

		if (!MessageHelper.loadedPrefs(portletSession, portletID)){
            if (debug){
    			System.out.println("Loading prefs....");
            }
			// load in the mappings from the portlet preferences.
			PortletPreferences prefs = request.getPreferences();
			Set availablePrefs = prefs.getMap().keySet();
			for (Iterator it=availablePrefs.iterator(); it.hasNext();){
				Object prefKey = it.next();
                if (debug){
				    System.out.println(" - "+prefKey.toString()+" ("+prefs.getValue(prefKey.toString(), "<blank>")+")");
                }
			}

			// get inputs
			int i = 1;
			String prefNamePrefix = PREF_IN_PREFIX+i;
			String name = null;
			String sourceName = null;
			String sourceNs = null;
            if (debug){
                System.out.println(" -- Find Inputs.");
			    System.out.println(" -- Looking for "+prefNamePrefix+PREF_NAME);
            }
			while (availablePrefs.contains(prefNamePrefix+PREF_NAME)){
				name = prefs.getValue(prefNamePrefix+PREF_NAME, null);
				if (name!=null){
                    if (debug){
					    System.out.println(" --- Looking for "+prefNamePrefix+PREF_SOURCENAME+" and "+prefNamePrefix+PREF_SOURCENS);
                    }
					sourceName = prefs.getValue(prefNamePrefix+PREF_SOURCENAME, name);
					sourceNs = prefs.getValue(prefNamePrefix+PREF_SOURCENS, NS_LOCAL);
					MessageBoxKey source = new MessageBoxKey(sourceName, sourceNs);
					if (NS_LOCAL.equals(sourceNs)){	
						source.setLocalNamespace(portletID);
					}
                    if (debug){
					    System.out.println(" --- Setting input mapping: "+portletID+", "+name+" => "+source);
                    }
					msgCentre.addInputMapping(portletID, name, source);
				}
				i++;
				prefNamePrefix = PREF_IN_PREFIX+i;
                if (debug){
				    System.out.println(" -- Looking for "+prefNamePrefix+PREF_NAME);
                }
			}

			// get outputs
			i = 1;
			prefNamePrefix = PREF_OUT_PREFIX+i;
            if (debug){
                System.out.println(" -- Find Outputs.");
    			System.out.println(" -- Looking for "+prefNamePrefix+PREF_NAME);
            }
			while (availablePrefs.contains(prefNamePrefix+PREF_NAME)){
				name = prefs.getValue(prefNamePrefix+PREF_NAME, null);
				if (name!=null){
					// add the basic (local) output mapping 
					MessageBoxKey target = new MessageBoxKey(name, portletID, true);
					//System.out.println("Found output mapping: "+portletID+", "+name);
					msgCentre.clearAllOutputMappings(target);

					// add any additional (global) output mappings 
					int j = 1;
					String prefPublishName = prefNamePrefix + PREF_PUBLISHNAME + j;
					String prefPublishNamespace = prefNamePrefix + PREF_PUBLISHNS + j;
					String publishName = null;
					String publishNs = null;
                    if (debug){
					    System.out.println(" -- Looking for "+prefPublishName);
                    }
					while (availablePrefs.contains(prefPublishName)){
						publishName = prefs.getValue(prefPublishName, null);
						if (publishName!=null){
							publishNs = prefs.getValue(prefPublishNamespace, "");
							target = new MessageBoxKey(publishName, publishNs);
                            if (debug){
							    System.out.println(" --- Adding output publishing to: "+portletID+", "+name+" => "+target);
                            }
							msgCentre.addOutputMapping(portletID, name, target);
						}
						j++;
						prefPublishName = prefNamePrefix + PREF_PUBLISHNAME + j;
						prefPublishNamespace = prefNamePrefix + PREF_PUBLISHNS + j;
                        if (debug){
						    System.out.println(" -- Looking for "+prefPublishName);
                        }
					}
				}
				i++;
				prefNamePrefix = PREF_OUT_PREFIX+i;
			}

			// finished loading mappings
			MessageHelper.setLoadedPrefs(portletSession, portletID, true);
		}
        if (debug){
    		System.out.println("Got prefs.");
        }
	}

	public static void savePrefs(PortletRequest request, String portletID, String sessionID){
        boolean debug = MessageConfig.debug();

		PortletSession portletSession = request.getPortletSession(true);
		PortletPreferences prefs = request.getPreferences();
		// first, clear any existing mappings
		MessageHelper.clearPrefs(prefs);

		// now put in the current mappings held for this portlet in the MessageCentre
		MessageCentre msgCentre = MessageHelper.getMessageCentre(portletSession, sessionID);

		// save input mappings
		Set inputKeys = msgCentre.getInputKeys(portletID);
		int i=1;
		String prefNamePrefix = null;
		for (Iterator it=inputKeys.iterator(); it.hasNext();){
			// save each input
			prefNamePrefix = PREF_IN_PREFIX+i;
			MessageBoxKey inputKey = (MessageBoxKey) it.next();
			MessageBoxKey sourceKey = msgCentre.findInputSource(inputKey);
			try {
				prefs.setValue(prefNamePrefix+PREF_NAME, inputKey.getName());
				prefs.setValue(prefNamePrefix+PREF_SOURCENAME, sourceKey.getName());
				String namespace = sourceKey.getNamespace();
				if (sourceKey.isLocal()){
					namespace = NS_LOCAL;
				}
				prefs.setValue(prefNamePrefix+PREF_SOURCENS, namespace);
			} catch (Exception e){ System.out.println("Error setting preference; "+e.getMessage()); e.printStackTrace(); }
			i++;
		}

		// save output mappings
		Set outputKeys = msgCentre.getOutputKeys(portletID);
		i = 1;
		for (Iterator it=outputKeys.iterator(); it.hasNext();){
			// save each output
			prefNamePrefix = PREF_OUT_PREFIX+i;
			MessageBoxKey outputKey = (MessageBoxKey) it.next();
			try {
                if (debug){
				    System.out.println("MessageHelper: saving pref output: "+prefNamePrefix+PREF_NAME+", "+outputKey.getName());
                }
				prefs.setValue(prefNamePrefix+PREF_NAME, outputKey.getName());
			} catch (Exception e){ System.out.println("Error setting preference; "+e.getMessage()); e.printStackTrace(); }

			// save additional output targets
			int j = 1;
			String prefPublishName = null;
			String prefPublishNamespace = null;
			Set targetKeys = msgCentre.findOutputTargets(outputKey);
			for (Iterator it2=targetKeys.iterator(); it2.hasNext();){
				MessageBoxKey targetKey = (MessageBoxKey) it2.next();
				prefPublishName = prefNamePrefix + PREF_PUBLISHNAME + j;
				prefPublishNamespace = prefNamePrefix + PREF_PUBLISHNS + j;
				if (!targetKey.isLocal()){
					try {
						if (debug){
                            System.out.println("MessageHelper: saving pref output target "+prefPublishName+": "+targetKey.getName()+" ("+targetKey.getNamespace()+")");
                        }
						prefs.setValue(prefPublishName, targetKey.getName());
						prefs.setValue(prefPublishNamespace, targetKey.getNamespace());
					} catch (Exception e){ System.out.println("Error setting preference; "+e.getMessage()); e.printStackTrace(); }
					j++;
				}
			}
			i++;
		}

		try {
			prefs.store();
		} catch (Exception e){ System.out.println("Error storing preferences; "+e.getMessage()); e.printStackTrace(); }

        if (debug){
		    System.out.println("MessageHelper: preferences saved");
        }
	}

	protected static void clearPrefs(PortletPreferences prefs){
		if (prefs==null) return;
		Set availablePrefs = prefs.getMap().keySet();

		// clear input mappings
		int i = 1;
		String prefNamePrefix = PREF_IN_PREFIX+i;
		while (availablePrefs.contains(prefNamePrefix+PREF_NAME)){
			try {
				prefs.reset(prefNamePrefix+PREF_NAME);
			} catch (Exception e) {}
			try {
				prefs.reset(prefNamePrefix+PREF_SOURCENAME);
			} catch (Exception e) {}
			try {
				prefs.reset(prefNamePrefix+PREF_SOURCENS);
			} catch (Exception e) {}

			i++;
			prefNamePrefix = PREF_IN_PREFIX+i;
		}

		// clear output mappings
		i = 1;
		prefNamePrefix = PREF_OUT_PREFIX+i;
		while (availablePrefs.contains(prefNamePrefix+PREF_NAME)){
			try {
				prefs.reset(prefNamePrefix+PREF_NAME);
			} catch (Exception e) {}

			// clear any additional (global) output mappings 
			int j = 1;
			String prefPublishName = prefNamePrefix + PREF_PUBLISHNAME + j;
			String prefPublishNamespace = prefNamePrefix + PREF_PUBLISHNS + j;
			while (availablePrefs.contains(prefPublishName)){
				try {
					prefs.reset(prefPublishName);
					prefs.reset(prefPublishNamespace);
				} catch (Exception e) { e.printStackTrace();}
				j++;
				prefPublishName = prefNamePrefix + PREF_PUBLISHNAME + j;
				prefPublishNamespace = prefNamePrefix + PREF_PUBLISHNS + j;
			}

			i++;
			prefNamePrefix = PREF_OUT_PREFIX+i;
		}

		try {
			prefs.store();
		} catch (Exception e){ System.out.println("Error storing preferences; "+e.getMessage()); e.printStackTrace(); }
	}

	/** Get message. If there is no message, returns null.
     */
	public Object get(String name){
		return msgCentre.get(portletID, name);
	}

	/** Get message as String. If there is no message, returns "".
     */
	public String getAsString(String name){
		if (msgCentre.get(portletID, name)!=null){
			return msgCentre.get(portletID, name).toString();
		}
		return "";
	}

	/** Send message. This will replace any message already in the box.
     */
	public void send(String name, Object message){
		msgCentre.send(portletID, name, message);
	}


}