/* Copyright 2005-2006 Michelle Osmond
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

package message;

import java.util.*;

import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;

/**
 * Some functions for use with the dynamic mapping form.
 * These could be put into a portlet's class/superclass instead.
 * 
 * @author Michelle Osmond
 * @version 1.1
 */
public class MessageUtils {

	public static String MSG = "mapMsg";

	/**
     * To be called in doEdit. Puts stuff in the request attributes for use by
     * edit_mappings.jsp; the doEdit code subsequently needs to include this jsp.
     */
	public static void prepareMappingForm(PortletRequest request, String portletID, String sessionID){
		// retrieve the mappings and available boxes
		MessageCentre centre = MessageHelper.getMessageCentre(request.getPortletSession(true), sessionID);
		Set inputKeys = centre.getInputKeys(portletID);
		Set outputKeys = centre.getOutputKeys(portletID);
		Set globalBoxKeys = centre.getPublicBoxKeys();
		Set availableSourceKeys = new TreeSet(outputKeys);
		availableSourceKeys.addAll(globalBoxKeys);

		// retrieve any message from a previously submitted form
		String mapMsg = request.getParameter(MessageUtils.MSG);
        if (mapMsg!=null && mapMsg.length()>0){
        	request.setAttribute(MessageUtils.MSG, mapMsg);
        }

		// prepare data for jsp
		request.setAttribute("portletID", portletID);
		request.setAttribute("messageCentre", centre);
		request.setAttribute("inputKeys", inputKeys);
		request.setAttribute("outputKeys", outputKeys);
		request.setAttribute("globalBoxKeys", globalBoxKeys);
		request.setAttribute("availableSourceKeys", availableSourceKeys);
	}

	/**
     * To be called in processAction when the edit_mappings.jsp form has been submitted.
     * Reads in the values and updates the MessageCentre, then saves to the portlet's preferences.
     */
	public static void processMappingForm(ActionRequest request, ActionResponse response, String portletID, String sessionID){

		try {
            boolean debug = MessageConfig.debug();
            if (debug){
    			System.out.println("Processing mapping form input...");
            }
	
			PortletSession portletSession = request.getPortletSession(true);
			Map params = request.getParameterMap();
	
            if (debug){
    			//debug: print out available parameters
    			for (Iterator it=params.keySet().iterator(); it.hasNext();){
    				Object key = it.next();
    				System.out.print(" - param:" +key+" = " );
    				String[] values = (String[]) params.get(key);
    				for (int i=0; i<values.length; i++){
    					System.out.print("["+values[i]+"]");
    				}
    				System.out.println();
    			}
			}
			
			/*
			 The form generates something like:
				 - param:inputName1 = [true[[WiredListing0.6488592238144903]]author]
				 - param:inputSource1 = [true[[WiredListing0.6488592238144903]]author]
				
				 - param:inputName2 = [true[[WiredListing0.6488592238144903]]book]
				 - param:inputSource2 = [true[[WiredListing0.6488592238144903]]book]
				
				 - param:outputName1 = [true[[WiredListing0.6488592238144903]]author]
				 - param:outputTargets1 = [false[[]]name][false[[]]title]
				 - param:newOutputTargetName1 = [author_name]
				 - param:newOutputTargetNamespace1 = []
				
				 - param:outputName2 = [true[[WiredListing0.6488592238144903]]book]
				 - param:outputTargets2 = [false[[]]title]
				 - param:newOutputTargetName2 = [book_title]
				 - param:newOutputTargetNamespace2 = []
			*/
	
			MessageCentre centre = MessageHelper.getMessageCentre(portletSession, sessionID);
	
			// look through the inputNameN/inputSourceN
			int i=1;
			while (params.containsKey("inputName"+i)){
				MessageBoxKey inputKey = MessageBoxKey.parseFormString(  request.getParameter("inputName"+i)  );
				MessageBoxKey inputSource = MessageBoxKey.parseFormString(  request.getParameter("inputSource"+i)  );
                if (debug){
	    			System.out.println(" -- Setting input mapping "+inputKey+" <= "+inputSource);
                }
				centre.addInputMapping(inputKey, inputSource);
				i++;
			}
	
	
			// look through the outputNameN/outputTargetsN/newOutputTargetN
			i=1;
			while (params.containsKey("outputName"+i)){
				MessageBoxKey outputKey = MessageBoxKey.parseFormString(  request.getParameter("outputName"+i)  );
                if (debug){
	    			System.out.println("Processing form for output mapping: "+outputKey);
                }
				centre.clearAllOutputMappings(outputKey);
				// add all specified output targets
				String[] targets = request.getParameterValues("outputTargets"+i);
				if (targets!=null){
					for (int j=0; j<targets.length; j++){
						MessageBoxKey targetKey = MessageBoxKey.parseFormString( targets[j] );
                        if (debug){
			    			System.out.println(" -- Setting output mapping: "+outputKey+" =>"+targetKey);
                        }
						centre.addOutputMapping(outputKey, targetKey);
					}
				}
				// additionally, see if a new target has been specified
				String newTargetName = request.getParameter("newOutputTargetName"+i);
				String newTargetNamespace = request.getParameter("newOutputTargetNamespace"+i);
				if (newTargetName!=null && newTargetName.trim().length()>0){
                    if (debug){
		    			System.out.println(" -- Setting new output mapping: "+outputKey+" =>"+newTargetName+", "+newTargetNamespace);
                    }
					centre.addOutputMapping(outputKey, new MessageBoxKey(newTargetName, newTargetNamespace));
				}
				i++;
			}
	
			// now save the preferences
            String msg_session_id = MessageHelper.getSessionID(request);
			MessageHelper.savePrefs(request, portletID, msg_session_id);
			// and reload them to be sure the session reflects the preferences
			MessageHelper.setLoadedPrefs(portletSession, portletID, false);
			MessageHelper.loadPrefs(request, portletID, msg_session_id);

            if (debug){	
    			System.out.println("Finished processing mapping form input.");
            }
			response.setRenderParameter(MessageUtils.MSG, "Message mappings updated.");

		} catch (Exception e){
			response.setRenderParameter(MessageUtils.MSG, "There was a problem with these mappings: "+e.getMessage());
			e.printStackTrace();
		}
	}


}