package message.store;

import java.util.*;
import javax.naming.Context;
import javax.naming.InitialContext;

import message.MessageBoxKey;
import message.MessageConfig;
import message.MessageStore;
import message.ejb.PortalMessageStore;
import message.ejb.PortalMessageStoreHome;

/**
 *  Storage and retrieval of portlet message mappings, and the messages. 
 *  They are stored on a remote EJB, which this class queries.
 *  This should be accessible to all portlet apps.
 * 
 * @author Michelle Osmond
 * @version 1.2
 */
public class EJBMessageStore implements MessageStore, java.io.Serializable {

    protected String sessionID;
	protected transient PortalMessageStore store;

	public EJBMessageStore(){ }

    // ----  Implement MessageStore methods. ------

    public void initialise(String sessionID){
        this.sessionID = sessionID;

        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
        env.put(Context.PROVIDER_URL, "localhost:1099");
        env.put("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");
    
        try {
            boolean debug = MessageConfig.debug();
            if (debug){
                System.out.println("EJBMessageStore getting PortalMessageStore bean for session id "+sessionID);
            }
            Context ctx = new InitialContext(env);
            Object obj = ctx.lookup( "PortalMessageStore" );
            PortalMessageStoreHome home = (PortalMessageStoreHome)javax.rmi.PortableRemoteObject.narrow( obj, PortalMessageStoreHome.class );
            try {
                if (debug){
                    System.out.println(" - looking for existing Store");
                }
                this.store = home.findByPrimaryKey(sessionID);
            } catch (Exception e){
                if (debug){
                    System.out.println(" - creating new Store");
                }
                this.store = home.create(sessionID);
            }
            if (debug){
                System.out.println("EJBMessageStore got PortalMessageStore bean for session id "+sessionID);
            }
        } catch ( Exception e ) {
            System.out.println( "Exception: " + e.getMessage() );
            e.printStackTrace();
        }
	}

    /** Check if the store is valid, and re-initialise if necessary & possible.
     *  (e.g. if session has been serialised, we may need to retrieve the EJB reference again.)
     */
    public boolean isValid(){    
        if (store==null && sessionID!=null){
            initialise(sessionID);
        }
        return store!=null && sessionID!=null;
    }

    public void saveInputMapping(MessageBoxKey key, MessageBoxKey source){
        try {
    		store.saveInputMapping(key, source);
        } catch (Exception e){ e.printStackTrace(); }
    }

    public void deleteInput(MessageBoxKey key){
        try {
    		store.deleteInput(key);
        } catch (Exception e){ e.printStackTrace(); }
    }

	public void saveOutputMapping(MessageBoxKey key, MessageBoxKey target){
        try {
    		store.saveOutputMapping(key, target);
        } catch (Exception e){ e.printStackTrace(); }
    }

	public void clearAllOutputMappings(MessageBoxKey key){
        try {
    		store.clearAllOutputMappings(key);
        } catch (Exception e){ e.printStackTrace(); }
    }

    public void deleteOutput(MessageBoxKey key){
        try {
    		store.deleteOutput(key);
        } catch (Exception e){ e.printStackTrace(); }
    }

    public Set findOutputTargets(MessageBoxKey key){
        try {
    		return store.findOutputTargets(key);
        } catch (Exception e){ e.printStackTrace(); return null;}
    }

    public MessageBoxKey getInputSource(MessageBoxKey key){
        try {
    		return store.getInputSource(key);
        } catch (Exception e){ e.printStackTrace(); return null; }
    }

    public Object getMessage(MessageBoxKey key){
        try {
    		return store.getMessage(key);
        } catch (Exception e){ e.printStackTrace(); return null; }
    }

    public void sendMessage(Set targetKeys, Object message){
        try {
    		store.sendMessage(targetKeys, message);
        } catch (Exception e){ e.printStackTrace(); }
    }

	public Set getInputKeys(String portletID){
        try {
    		return store.getInputKeys(portletID);
        } catch (Exception e){ e.printStackTrace(); return null; }
    }

	public Set getOutputKeys(String portletID){
        try {
    		return store.getOutputKeys(portletID);
        } catch (Exception e){ e.printStackTrace(); return null; }
    }

    public Map getBoxes(){
        try {
    		return store.getBoxes();
        } catch (Exception e){ e.printStackTrace(); return null; }
    }

    public Set getPublicBoxKeys(){
        try {
    		return store.getPublicBoxKeys();
        } catch (Exception e){ e.printStackTrace(); return null; }
    }

	public Set getAllInputKeys(){
        try {
    		return store.getAllInputKeys();
        } catch (Exception e){ e.printStackTrace(); return null; }
    }

	public Set getAllOutputKeys(){
        try {
    		return store.getAllOutputKeys();
        } catch (Exception e){ e.printStackTrace(); return null; }
    }

}